#!/usr/bin/python
# @author Miroslav Bodis 2014
# miroslavbodis.com

# I M P O R T A N T N 
# dont forget to run: $ sudo modprobe bcm2835-v4l2

import math  
import cv
import time
import Image
 
import RPi.GPIO as GPIO
from time import sleep
import pygame, sys, time, random
from pygame.locals import *
from array import array
import sys, getopt

#screen resolution
SCR_W = 320
SCR_H = 240

# horizontal/vertical percentage limit
PERCT_H = 20
PERCT_V = 25 

moving_H = False
moving_V = False

#GPIO board setup
GPIO.setmode(GPIO.BOARD)

#LED
GPIO.setup(11, GPIO.OUT)
GPIO.output(11, 1)
GPIO.setup(12, GPIO.OUT)
GPIO.output(12, 1)

#MOTOR-1 (jaws)
GPIO.setup(13, GPIO.OUT)
GPIO.output(13, 1)
GPIO.setup(15, GPIO.OUT)
GPIO.output(15, 1)

#MOTOR-2 (upper joint)
GPIO.setup(16, GPIO.OUT)
GPIO.output(16, 1)
GPIO.setup(18, GPIO.OUT)
GPIO.output(18, 1)

#MOTOR-3 (middle joint)
GPIO.setup(22, GPIO.OUT)
GPIO.output(22, 1)
GPIO.setup(31, GPIO.OUT)
GPIO.output(31, 1)

#MOTOR-4 (bottom joint)
GPIO.setup(32, GPIO.OUT)
GPIO.output(32, 1)
GPIO.setup(33, GPIO.OUT)
GPIO.output(33, 1)

#MOTOR-5 (base right left)
GPIO.setup(35, GPIO.OUT)
GPIO.output(35, 1)
GPIO.setup(36, GPIO.OUT)
GPIO.output(36, 1)

def safeExit():
    GPIO.cleanup()

def DetectFace(image):

    # flip vertically
    cv.ConvertImage(image, image, cv.CV_CVTIMG_FLIP)

    # convert to grayscale 
    cv.CvtColor(image, grayscale, cv.CV_BGR2GRAY)

    # Scale input image for faster processing
    cv.Resize(grayscale, smallImage, cv.CV_INTER_LINEAR)

    # Equalize the histogram
    #cv.EqualizeHist(smallImage, smallImage)

    # Detect the faces
    faces = cv.HaarDetectObjects(
            smallImage, faceCascade, cv.CreateMemStorage(0),
            haar_scale, min_neighbors, haar_flags, min_size
        )

     
    # If faces are found
    if faces:
        print "---- face detected ----"
        GPIO.output(12, 0)
        for ((x, y, w, h), n) in faces:
            # the input to cv.HaarDetectObjects was resized, so scale the
            # bounding box of each face and convert it to two CvPoints
            pt1 = (int(x * image_scale), int(y * image_scale))
            pt2 = (int((x + w) * image_scale), int((y + h) * image_scale))
            cv.Rectangle(image, pt1, pt2, cv.RGB(255, 0, 0), 5, 8, 0)       

            # movement owi hand
            h_chk = horizontal_check(int(x * image_scale), int(y * image_scale), int(w * image_scale), int(h * image_scale), SCR_W, SCR_H, PERCT_H)
            move_owi_horizontal(h_chk)
            v_chk=vertical_check( int(x * image_scale), int(y * image_scale), int(w * image_scale), int(h * image_scale), SCR_W, SCR_H, PERCT_V)
            move_owi_vertical(v_chk)
    else:
        print "---- no ----"
        GPIO.output(12, 1)
        move_owi_horizontal(0)
        move_owi_vertical(0)
 
    return image
 

def horizontal_check(x, y, w, h, src_w, src_h, limit):
    global moving_H

    dd = distance_two_points(int(x+w/2), int(src_h/2), int(src_w/2), int(src_h/2))
    pp = distance_two_point_in_percentage(0, int(src_h/2), int(src_w/2), int(src_h/2), dd)
    #print "horizontal distance from center: " + str(pp) + " %"

    # if in move and close to center
    if (moving_H and  (pp<(limit*2)) ):
        print "/// Horizotal lock ///"
        return 0

    if (pp > limit):
        if (int(x+w/2) > int(src_w/2)):
            return 1
        else:
            return -1
    else:
        return 0
    

def vertical_check(x, y, w, h, src_w, src_h, limit):
    global moving_V

    dd = distance_two_points(int(src_w/2), int(y+h/2), int(src_w/2), int(src_h/2))
    pp = distance_two_point_in_percentage(int(src_w/2), 0, int(src_w/2), int(src_h/2), dd)
    #print "vertical distance from center: " + str(pp) + " %"

    # if in move and close to center
    if (moving_V and  (pp<(limit*2)) ):
        print "/// Vertical lock ///"
        return 0

    if (pp > limit):
        if (int(y+h/2) < int(src_h/2)):
            return 1
        else:
            return -1
    else:
        return 0


def distance_two_point_in_percentage(x1, y1, x2, y2, dist):
    d1 = distance_two_points(x1, y1, x2, y2)
    d2 = dist
    return float(d2/d1*100)

def distance_two_points(x1, y1, x2, y2):
    return math.sqrt((x1-x2) * (x1-x2) + (y1-y2)*(y1-y2))
    

# / -1 == rotate_left / 0 == stay / +1 == rotate_right /
def move_owi_horizontal(state):
    global moving_H    
    
    if (state == -1):
        print "rotate left"
        GPIO.output(33, 1)
        GPIO.output(32, 0)        
        moving_H = True

    elif (state == 0):
        #print "rotate stop"
        GPIO.output(33, 1)
        GPIO.output(32, 1)
        moving_H = False

    elif (state == 1):
        print "rotate right" 
        GPIO.output(32, 1)
        GPIO.output(33, 0)
        moving_H = True

# / -1 == move down / 0 == stay / +1 == move up /
def move_owi_vertical(state):
    global moving_V

    if (state == -1):
        print "move down"
        #GPIO.output(36, 0)
        #GPIO.output(22, 0)
        GPIO.output(16,1)
        GPIO.output(18,0)
        moving_V = True

    elif (state == 0):
        #print "move stop"
        #GPIO.output(35, 1)
        #GPIO.output(36, 1)
        #GPIO.output(22, 1)
        #GPIO.output(31, 1)
        GPIO.output(16, 1)
        GPIO.output(18, 1)
        moving_V = False

    elif (state == 1):
        print "move up"
        #GPIO.output(35, 0)
        #GPIO.output(31, 0)
        GPIO.output(18, 1)
        GPIO.output(16, 0)
        moving_V = True


#----------
# M A I N
#----------

#capture = cv.CaptureFromFile("test.avi")
capture = cv.CaptureFromCAM(-1)
cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_WIDTH, SCR_W)
cv.SetCaptureProperty(capture, cv.CV_CAP_PROP_FRAME_HEIGHT, SCR_H)

#faceCascade = cv.Load("data/haarcascade_frontalface_default.xml")
faceCascade = cv.Load("/usr/local/lib/python2.7/dist-packages/SimpleCV/Features/HaarCascades/face.xml")
#faceCascade = cv.Load("/usr/local/lib/python2.7/dist-packages/SimpleCV/Features/HaarCascades/face2.xml")
#faceCascade = cv.Load("/usr/local/lib/python2.7/dist-packages/SimpleCV/Features/HaarCascades/face3.xml")

min_size = (20,20) #20, 20
image_scale = 2
haar_scale = 4 #1.1
min_neighbors = 3
haar_flags = 0

grayscale = cv.CreateImage((SCR_W, SCR_H), 8, 1)
smallImage = cv.CreateImage((cv.Round(SCR_W / image_scale), cv.Round(SCR_H / image_scale)), 8 ,1)

while (cv.WaitKey(15)==-1):
   
    # get image
    img = cv.QueryFrame(capture)

    # detect faces
    img = DetectFace(img)
    
    #show as video   
    cv.ShowImage("face detection test", img)
    

cv.ReleaseCapture(capture)

