#!/usr/bin/env python
# @author Miroslav Bodis 2014
# miroslavbodis.com

import RPi.GPIO as GPIO
from time import sleep
import pygame, sys, time, random
from pygame.locals import *
from array import array
import sys, getopt # input argumnets

pygame.init()
pygame.key.set_repeat(100, 100)
gameArea = pygame.display.set_mode((300, 300))

GPIO.setmode(GPIO.  BOARD)

#GPIO.setup(7, GPIO.OUT)  dead
#GPIO.output(7, 1)        dead
#GPIO.setup(29, GPIO.OUT) dead ? 
#GPIO.output(29, 1)       dead ?

## ## ## ## INIT ## ## ## ## ##

#LED
GPIO.setup(11, GPIO.OUT)
GPIO.output(11, 1)
GPIO.setup(12, GPIO.OUT)
GPIO.output(12, 1)

#MOTOR-1 (jaws)
GPIO.setup(13, GPIO.OUT)
GPIO.output(13, 1)
GPIO.setup(15, GPIO.OUT)
GPIO.output(15, 1)

#MOTOR-2 (upper joint)
GPIO.setup(16, GPIO.OUT)
GPIO.output(16, 1)
GPIO.setup(18, GPIO.OUT)
GPIO.output(18, 1)

#MOTOR-3 (middle joint)
GPIO.setup(22, GPIO.OUT)
GPIO.output(22, 1)
GPIO.setup(31, GPIO.OUT)
GPIO.output(31, 1)

#MOTOR-4 (bottom joint)
GPIO.setup(32, GPIO.OUT)
GPIO.output(32, 1)
GPIO.setup(33, GPIO.OUT)
GPIO.output(33, 1)

#MOTOR-5 (base right left)
GPIO.setup(35, GPIO.OUT)
GPIO.output(35, 1)
GPIO.setup(36, GPIO.OUT)
GPIO.output(36, 1)

FILE_NAME = "default_keyframes.txt"
if len(sys.argv) > 1:
    FILE_NAME = sys.argv[1]
file = open(FILE_NAME, 'r')


run = True # recover owi back to default position
lock = [0,0,0,0,0,0]
savedInterval = array('d', [0,0,0,0,0,0]) #store changes from file
actualPosition = array('d', [0,0,0,0,0,0]) #store changes from start position
stepStart = int(round(time.time() * 1000))

def safeExit():
    GPIO.cleanup()
    file.close()

# test if owi reached cuurent keyFrame
def isOnPosition():
    for  i in range(0,6):        
        if (savedInterval[i] != 0):
            return False
    return True

def setMotorToPosition(index, GPIO_A, GPIO_B):
    if (savedInterval[index] != 0 and abs(savedInterval[index]) > interval):
        if savedInterval[index] > 0 and lock[index] == 0:                
            GPIO.output(GPIO_B, 0)
            lock[index] = 1            
        elif savedInterval[index] < 0 and lock[index] == 0:
            GPIO.output(GPIO_A, 0)
            lock[index] = 1            
    elif savedInterval[index] != 0 and abs(savedInterval[index]) <= interval and (lock[index] == 1 or lock[index] == 1):
        GPIO.output(GPIO_A, 1)
        GPIO.output(GPIO_B, 1)
        savedInterval[index] = 0
        lock[index] = 0        

# read next position (6 lines) from file
def readNextKeyFrame():
    print "readNextKeyFrame"
    
    for i in range(0,7):
        line = file.readline()
        
        if len(line) == 0:
            return False
        
        #print line
            
        if i < 6:            
            savedInterval[i] = float(line)
        elif i == 6:
            if line.strip() == '#':
                print " - - ok - - "
            elif line.strip() == '##':
                print "EOF"                
            else:
                return False                
    updateActualPosition()
    printKeyFrame()  

# print current keyFrame
def printKeyFrame():
    print "printKeyFrame"
    for i in range(0,6):
        print savedInterval[i]  

# log your current positon, so you can recover back to initial state
def updateActualPosition():
    print "updateActualPosition"
    for  i in range(0,6):        
        actualPosition[i] += savedInterval[i]
    
# set new location from actual position to owi to move
def reloadPosition():
    print "reloadPosition"
    for  i in range(0,6):        
        savedInterval[i] = actualPosition[i]


## ## ## ## RUN PROGRAM ## ## ## ## ##
        
try:
    
    stepStart = int(round(time.time() * 1000))
    readNextKeyFrame();    
    while run:        
        if isOnPosition():
            stepStart = int(round(time.time() * 1000))
            r = readNextKeyFrame();
            if r == False:
                break
                        
        interval = int(round(time.time() * 1000)) - stepStart
        #print interval

        setMotorToPosition(0, 12, 11)        
        setMotorToPosition(1, 15, 13)
        setMotorToPosition(2, 18, 16)
        setMotorToPosition(3, 31, 22)
        setMotorToPosition(4, 33, 32)
        setMotorToPosition(5, 36, 35)            
        sleep(0.01)
        
        
    print "sleep 3 sec"
    sleep(3)
    print "restoring to default position"
    reloadPosition()
    
    stepStart = int(round(time.time() * 1000))    
    while run:        

        if isOnPosition():
            break
        interval = int(round(time.time() * 1000)) - stepStart
        #print interval

        setMotorToPosition(0, 11, 12)        
        setMotorToPosition(1, 13, 15)
        setMotorToPosition(2, 16, 18)
        setMotorToPosition(3, 22, 31)
        setMotorToPosition(4, 32, 33)
        setMotorToPosition(5, 35, 36)
        
        
            
        sleep(0.01)    

    print "OWI is back in init state"

    safeExit()
except KeyboardInterrupt:    
    safeExit()

