#!/usr/bin/env python
# @author Miroslav Bodis 2014
# miroslavbodis.com

import RPi.GPIO as GPIO
from time import sleep
import pygame, sys, time, random
from pygame.locals import *
from array import array
import sys, getopt # argumnets

pygame.init()
pygame.key.set_repeat(100, 100)
gameArea = pygame.display.set_mode((300, 300))

#print GPIO.VERSION
#exit()

GPIO.setmode(GPIO.  BOARD)

#GPIO.setup(7, GPIO.OUT)  dead
#GPIO.output(7, 1)        dead
#GPIO.setup(29, GPIO.OUT) dead
#GPIO.output(29, 1)       dead

## ## ## ## INIT ## ## ## ## ##

#LED
GPIO.setup(11, GPIO.OUT)
GPIO.output(11, 1)
GPIO.setup(12, GPIO.OUT)
GPIO.output(12, 1)

#MOTOR-1 (jaws)
GPIO.setup(13, GPIO.OUT)
GPIO.output(13, 1)
GPIO.setup(15, GPIO.OUT)
GPIO.output(15, 1)

#MOTOR-2 (upper joint)
GPIO.setup(16, GPIO.OUT)
GPIO.output(16, 1)
GPIO.setup(18, GPIO.OUT)
GPIO.output(18, 1)

#MOTOR-3 (middle joint)
GPIO.setup(22, GPIO.OUT)
GPIO.output(22, 1)
GPIO.setup(31, GPIO.OUT)
GPIO.output(31, 1)

#MOTOR-4 (bottom joint)
GPIO.setup(32, GPIO.OUT)
GPIO.output(32, 1)
GPIO.setup(33, GPIO.OUT)
GPIO.output(33, 1)

#MOTOR-5 (base right left)
GPIO.setup(35, GPIO.OUT)
GPIO.output(35, 1)
GPIO.setup(36, GPIO.OUT)
GPIO.output(36, 1)

FILE_NAME = "default_keyframes.txt"
if len(sys.argv) > 1:
    FILE_NAME = sys.argv[1]
file = open(FILE_NAME, 'w')

run = True
runDefault = False # recover owi back to default position
lock = [0,0,0,0,0,0]
lockChr = ['','','','','','']
timePressed =  array('d', [0,0,0,0,0,0])
pressedInterval = array('d', [0,0,0,0,0,0]) #store changes from previous position to current 
actualPosition = array('d', [0,0,0,0,0,0]) #store changes from start position
recoveryStart = int(round(time.time() * 1000))

## on key down event toggle relay ##
def keyDownLocker(key, char_A, GPIO_A, char_B, GPIO_B, index):
    if key == ord(char_A):
        if lock[index] == 0:
            GPIO.output(GPIO_A, 0)
            lock[index] = 1
            lockChr[index] = char_A
            timePressed[index] = int(round(time.time() * 1000))
    if key == ord(char_B):
        if lock[index] == 0:
            GPIO.output(GPIO_B, 0)
            lock[index] = 1
            lockChr[index] = char_B
            timePressed[index] = int(round(time.time() * 1000))


## on key up event toggle relay ##
def keyUpLocker(key, char_A, GPIO_A, char_B, GPIO_B, index):
    if key == ord(char_A):
        if lock[index] == 1 and lockChr[index] == char_A:
            GPIO.output(GPIO_A, 1)
            lock[index] = 0            
            pressedInterval[index] += ( int(round(time.time() * 1000)) -  timePressed[index])
            actualPosition[index] += ( int(round(time.time() * 1000)) -  timePressed[index])
            printActualState()
            
    if key == ord(char_B) and lockChr[index] == char_B:
        if lock[index] == 1:
            GPIO.output(GPIO_B, 1)
            lock[index] = 0            
            pressedInterval[index] += ( int(round(time.time() * 1000)) -  timePressed[index]) * -1
            actualPosition[index] += ( int(round(time.time() * 1000)) -  timePressed[index]) * -1
            printActualState()



## print help
def printHelp():
    print "# # # # # #"
    print "CONTROLS:"
    print " "
    print "q - toggle LED"
    print "w/s jaws controll (M1)"
    print "e/d first upper joint (M2)"
    print "r/f second middle joint (M3)"
    print "t/g bottom joint (M4)"
    print "y/h base left/right (M5)"
    print " "
    print "enter => save actulal positon as key frame"
    print "backspace => exit with restoring default state"
    print "esc => exit"
    print "# # # # # #"

def printActualState():
    print "# # # ACTUAL STATE # # #"    
    for i in range(0,6):        
        print pressedInterval[i]
    
def saveActualPosition():
    print "# # # KEYFRAME SAVED # # #"
    printActualState()    
    for  i in range(0,6):        
        file.write(str(pressedInterval[i]))
        file.write("\n")    
        pressedInterval[i] = 0     
    file.write("#\n")        

def safeExit():
    GPIO.cleanup()
    file.close()

def isBackToDefaultPosition():
    for  i in range(0,6):        
        if (actualPosition[i] != 0):
            return False
    return True

def returnMotorToDefaultPosition(index, GPIO_A, GPIO_B):
    if (actualPosition[index] != 0 and abs(actualPosition[index]) > interval):
        if actualPosition[index] > 0 and lock[index] == 0:                
            GPIO.output(GPIO_B, 0)
            lock[index] = 1            
        elif actualPosition[index] < 0 and lock[index] == 0:
            GPIO.output(GPIO_A, 0)
            lock[index] = 1            
    elif actualPosition[index] != 0 and abs(actualPosition[index]) <= interval and (lock[index] == 1 or lock[index] == 1):
        GPIO.output(GPIO_A, 1)
        GPIO.output(GPIO_B, 1)
        actualPosition[index] = 0
        lock[index] = 0        

## ## ## ## RUN PROGRAM ## ## ## ## ##
    
printHelp()
try:
    while run:        
        for event in pygame.event.get():
            
            if event.type == KEYDOWN:
                #print event.key
                                                      
                if event.key == ord('q') or event.key == ord('a'):    
                    keyDownLocker(event.key, 'q', 11, 'a', 12, 0)
                if event.key == ord('w') or event.key == ord('s'):    
                    keyDownLocker(event.key, 'w', 13, 's', 15, 1)
                if event.key == ord('e') or event.key == ord('d'):    
                    keyDownLocker(event.key, 'e', 16, 'd', 18, 2)
                if event.key == ord('r') or event.key == ord('f'):    
                    keyDownLocker(event.key, 'r', 22, 'f', 31, 3)
                if event.key == ord('t') or event.key == ord('g'):    
                    keyDownLocker(event.key, 't', 32, 'g', 33, 4)
                if event.key == ord('y') or event.key == ord('h'):    
                    keyDownLocker(event.key, 'y', 35, 'h', 36, 5)
                
                if event.key == 27: # ESC - exit
                    run = False      
                if event.key == 13: # ENTER    
                    saveActualPosition()
                if event.key == 8: #BACKSPACE - exit retur to default position
                    run = False
                    runDefault = True                                
                    
            elif event.type == KEYUP:
                if event.key == ord('q') or event.key == ord('a'):
                    keyUpLocker(event.key, 'q', 11, 'a', 12, 0)
                if event.key == ord('w') or event.key == ord('s'):
                    keyUpLocker(event.key, 'w', 13, 's', 15, 1)
                if event.key == ord('e') or event.key == ord('d'):
                    keyUpLocker(event.key, 'e', 16, 'd', 18, 2)
                if event.key == ord('r') or event.key == ord('f'):
                    keyUpLocker(event.key, 'r', 22, 'f', 31, 3)
                if event.key == ord('t') or event.key == ord('g'):
                    keyUpLocker(event.key, 't', 32, 'g', 33, 4)
                if event.key == ord('y') or event.key == ord('h'):
                    keyUpLocker(event.key, 'y', 35, 'h', 36, 5)

    for  i in range(0,6):        
        lock[i] = 0
        
    recoveryStart = int(round(time.time() * 1000))
    
    while runDefault:
        if isBackToDefaultPosition():
            runDefault = False
        interval = int(round(time.time() * 1000)) - recoveryStart
        #print interval

        returnMotorToDefaultPosition(0, 11, 12)        
        returnMotorToDefaultPosition(1, 13, 15)
        returnMotorToDefaultPosition(2, 16, 18)
        returnMotorToDefaultPosition(3, 22, 31)
        returnMotorToDefaultPosition(4, 32, 33)
        returnMotorToDefaultPosition(5, 35, 36)
            
        sleep(0.01)
        
    safeExit()
except KeyboardInterrupt:    
    safeExit()

