#!/usr/bin/python
# @author Miroslav Bodis 2014
# miroslavbodis.com

import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web
import tornado.websocket

import RPi.GPIO as GPIO
from time import sleep
import pygame, sys, time, random
from pygame.locals import *
from array import array
import sys, getopt


GPIO.setmode(GPIO.BOARD)
#LED
GPIO.setup(11, GPIO.OUT)
GPIO.output(11, 1)
GPIO.setup(12, GPIO.OUT)
GPIO.output(12, 1)

#MOTOR-1 (jaws)
GPIO.setup(13, GPIO.OUT)
GPIO.output(13, 1)
GPIO.setup(15, GPIO.OUT)
GPIO.output(15, 1)

#MOTOR-2 (upper joint)
GPIO.setup(16, GPIO.OUT)
GPIO.output(16, 1)
GPIO.setup(18, GPIO.OUT)
GPIO.output(18, 1)

#MOTOR-3 (middle joint)
GPIO.setup(22, GPIO.OUT)
GPIO.output(22, 1)
GPIO.setup(31, GPIO.OUT)
GPIO.output(31, 1)

#MOTOR-4 (bottom joint)
GPIO.setup(32, GPIO.OUT)
GPIO.output(32, 1)
GPIO.setup(33, GPIO.OUT)
GPIO.output(33, 1)

#MOTOR-5 (base right left)
GPIO.setup(35, GPIO.OUT)
GPIO.output(35, 1)
GPIO.setup(36, GPIO.OUT)
GPIO.output(36, 1)


run=False
savedInterval = array('d', [0,0,0,0,0,0]) #new changes from via message
stepStart = int(round(time.time() * 1000))

def safeExit():
    GPIO.cleanup()
   

# test if owi reached cuurent keyFrame
def isOnPosition():
    for  i in range(0,6):        
        if (savedInterval[i] != 0):
            return False
    return True

# toggle GPIO by interval and savedIndex
def setMotorToPosition(index, GPIO_A, GPIO_B, interval):
    if (savedInterval[index] != 0 and abs(savedInterval[index]) > interval):
        if savedInterval[index] > 0:                
            GPIO.output(GPIO_B, 0)                
        elif savedInterval[index] < 0:
            GPIO.output(GPIO_A, 0)           
    elif savedInterval[index] != 0 and abs(savedInterval[index]) <= interval:
        GPIO.output(GPIO_A, 1)
        GPIO.output(GPIO_B, 1)
        savedInterval[index] = 0        

# print current keyFrame
def printKeyFrame():
    print "printKeyFrame"
    for i in range(0,6):
        print savedInterval[i]  

# execute movements
def move_owi(index, t):   
    savedInterval[index] = t

    run=True
    stepStart = int(round(time.time() * 1000))

    while run:
        # test if all movements has been done
        if isOnPosition():
            run=False
            break
             
        i = int(round(time.time() * 1000)) - stepStart

        setMotorToPosition(0, 12, 11, i)   
        setMotorToPosition(1, 15, 13, i)
        setMotorToPosition(2, 18, 16, i)
        setMotorToPosition(3, 31, 22, i)
        setMotorToPosition(4, 33, 32, i)
        setMotorToPosition(5, 36, 35, i)            
        sleep(0.01)

class OwiHandler(tornado.websocket.WebSocketHandler):
    def open(self):
        print "Connection opened"
        self.write_message("Connection opened")

    def on_close(self):
        print "Connection closed"

    def on_message(self, message):
        print "Message received ()".format(message)
        print message
        self.write_message("Message received!")

        if "#" in message:
	    str = message.split("#")           
            move_owi(int(str[0]),float(str[1]))

try:
    if __name__ == "__main__":
        tornado.options.parse_command_line()
        app = tornado.web.Application(handlers=[(r"/", OwiHandler)])
        server = tornado.httpserver.HTTPServer(app)
        server.listen(8000)
        tornado.ioloop.IOLoop.instance().start()
except KeyboardInterrupt:
    safeExit()
